@@ -27,6 +27,7 @@
 #include "msettingsdialog.h"
 #include "mstatstab.h"
 #include "msystemtray.h"
+#include "progressbar.h"
 #include "systemconfig.h"
 
 #include <qdatetimeedit.h>
@@ -42,6 +43,8 @@
 #include <kaction.h>
 #include <kdebug.h>
 #include <kiconloader.h>
+#include <kglobalaccel.h>
+#include <kkeydialog.h>
 #include <klocale.h>
 #include <kmenubar.h>
 #include <kmessagebox.h>
@@ -56,9 +59,8 @@ MMainWindow::MMainWindow()
 	// Iface
 	: DCOPObject("KShutdownIface"),
 	KMainWindow(0, "MMainWindow", WStyle_ContextHelp),
+	_showMinimizeInfo(true),
 	_oldSec(-1),
-	_messageDialog(0),
-	_stats(0),
 	_delayType(DelayType_Now)
 {
 	_instance = this;
@@ -74,6 +76,7 @@ MMainWindow::MMainWindow()
 
 	// init widgets
 	setCentralWidget(initWidgets());
+	initActions();
 	initMainMenu();
 
 	// init DCOP
@@ -94,7 +97,7 @@ MMainWindow::MMainWindow()
 
 MMainWindow::~MMainWindow()
 {
-// TODO: 0.9: remember/restore time
+// TODO: 2.0: remember/restore time
 	kshutdownrc->generalAction = ks_actions->current();
 	kshutdownrc->generalDelayType = cb_delayTypes->currentItem();
 	kshutdownrc->writeConfig();
@@ -165,38 +168,19 @@ void MMainWindow::setTime(const QTime &time)
 	_dateTimeEdit->timeEdit()->setTime(QTime(time.hour(), time.minute(), 0));
 }
 
-void MMainWindow::messageDialog(const int timeout)
-{
-	if (_messageDialog)
-		delete _messageDialog;
-
-	KWin::setOnDesktop(winId(), KWin::currentDesktop());
-	// show warning message dialog
-	_messageDialog = new MMessageDialog(
-		timeout,
-		ks_actions->current(),
-		ks_actions->getCurrentName()
-	);
-	_messageDialog->exec();
-	delete _messageDialog;
-	_messageDialog = 0;
-
-	return;
-}
-
 void MMainWindow::setAction(const Action::Type action)
 {
 	// kdDebug() << "MMainWindow::setAction: " << ks_actions->getName(action) << endl;
 
 	if (!ks_actions->isEnabled(action))
 	{
-// FIXME: 0.9: "Start" button incorectly disabled/enabled after config change
-// FIXME: 0.9: confusing
-		b_startStop->setEnabled(false);
+// FIXME: 2.0: "Start" button incorectly disabled/enabled after config change
+// FIXME: 2.0: confusing
+		_startAction->setEnabled(false);
 
 		return;
 	}
-	b_startStop->setEnabled(true);
+	_startAction->setEnabled(true);
 
 	ks_actions->setCurrent(action);
 	cb_actions->setCurrentItem(action - 1);
@@ -220,8 +204,7 @@ void MMainWindow::setDate(const QDate &date)
 	_dateTimeEdit->dateEdit()->setDate(date);
 }
 
-void MMainWindow::start(const bool confirmation)
-{
+void MMainWindow::start(const bool confirmation) {
 	if (ks_actions->active())
 		return;
 
@@ -236,9 +219,7 @@ void MMainWindow::start(const bool confirmation)
 	{
 		KMessageBox::information(
 			this,
-			"<qt>" +
-			i18n("Click the <b>Select a command...</b> button first.") +
-			"</qt>"
+			MiscUtils::HTML(i18n("Click the <b>Select a command...</b> button first."))
 		);
 
 		return;
@@ -246,7 +227,7 @@ void MMainWindow::start(const bool confirmation)
 
 	if (confirmation)
 	{
-		if (!Confirmation::confirm(ks_actions->current()))
+		if (!Confirmation::confirm(ks_actions->current(), getTimeInfo()))
 			return; // action not confirmed by user; do nothing
 	}
 
@@ -275,9 +256,6 @@ void MMainWindow::start(const bool confirmation)
 
 	_oldSec = -1;
 
-	b_startStop->setIconSet(SmallIcon("stop"));
-	b_startStop->setText(i18n("Stop [Esc]"));
-
 	if (MSystemTray::isInstance())
 		ks_tray->setActive(true);
 
@@ -289,22 +267,28 @@ void MMainWindow::start(const bool confirmation)
 		cancel();
 		KMessageBox::error(
 			0,
-			"<qt>" +
-			i18n(
-				"Selected date/time is earlier " \
-				"than current date/time!"
-			) +
-			"<br><br>" +
-			selDT + "<br>" +
-			curDT + "<br>" \
-			"<br>" +
-			i18n("Action cancelled!") +
-			"</qt>"
+			MiscUtils::HTML(
+				i18n("Selected date/time is earlier than current date/time!") +
+				"<br><br>" +
+				selDT + "<br>" +
+				curDT + "<br>" \
+				"<br>" +
+				i18n("Action cancelled!")
+			)
 		);
 
 		return;
 	}
 
+	int secsTo = dt_start.secsTo(dt_end);
+
+	// setup progress bar
+	if (kshutdownrc->progressBarEnabled) {
+		ProgressBar *progressBar = ProgressBar::getInstance();
+		progressBar->setValues(0, secsTo);
+		progressBar->show();
+	}
+
 	if (_delayType == DelayType_OnAppExit)
 	{
 		_timer->start(2000);
@@ -313,14 +297,13 @@ void MMainWindow::start(const bool confirmation)
 	else
 	{
 		_timer->start(500); // this timer value is good ..
-		updateTimeInfo(dt_start.secsTo(dt_end));
+		updateTimeInfo(secsTo);
 	}
 }
 
 void MMainWindow::cancel()
 {
-	if (_messageDialog)
-		_messageDialog->slotCancel();
+	MMessageDialog::cancel();
 
 	if (!ks_actions->active() && (_delayType != DelayType_OnAppExit))
 		return;
@@ -332,13 +315,13 @@ void MMainWindow::cancel()
 
 	_timer->stop();
 	setCaptions();
-	b_startStop->setIconSet(SmallIcon("button_ok"));
-	b_startStop->setText(i18n("Start [Enter]"));
 	if (MSystemTray::isInstance())
 		ks_tray->setActive(false);
 
+	ProgressBar::freeInstance();
+
 	setWidgetsEnabled(true);
-// TODO: 0.9: MiscUtils::customMessage(i18n("Action cancelled!"));
+// TODO: 2.0: MiscUtils::customMessage(i18n("Action cancelled!"));
 }
 
 bool MMainWindow::shutDown()
@@ -369,9 +352,13 @@ bool MMainWindow::logout()
 
 void MMainWindow::configure()
 {
-	MSettingsDialog *dialog = new MSettingsDialog(this);
-	dialog->exec();
-	delete dialog;
+	if (MSettingsDialog::isActive())
+		return;
+
+	new MSettingsDialog(this);
+	MSettingsDialog::freeInstance();
+
+	_showMinimizeInfo = true;
 }
 
 QString MMainWindow::getStatusInfo()
@@ -379,21 +366,33 @@ QString MMainWindow::getStatusInfo()
 	return caption();
 }
 
+void MMainWindow::makeInvisible() {
+	if (MSystemTray::isInstance())
+		ks_tray->hide();
+	hide();
+}
+
 void MMainWindow::makeVisible()
 {
+	MSystemTray::setMode((MSystemTray::Mode)kshutdownrc->systemTray);
 	show();
-	KWin::setOnDesktop(winId(), KWin::currentDesktop());
+	WId id = winId();
+	KWin::setOnDesktop(id, KWin::currentDesktop());
+	KWin::deIconifyWindow(id);
 	raise();
 }
 
 void MMainWindow::setTestMode(bool yes)
 {
+	if (ks_actions->testMode() == yes)
+		return;
+
 	ks_actions->setTestMode(yes);
 	setCaptions(); // add/remove "[ TEST MODE ]" to/from window caption.
 	MiscUtils::customMessage(ks_actions->testMode() ? i18n("Test mode enabled") : i18n("Test mode disabled"));
 }
 
-void MMainWindow::calcSelectedTime()
+QString MMainWindow::getTimeInfo()
 {
 	// start time = now
 	dt_start.setDate(QDate::currentDate());
@@ -425,6 +424,14 @@ void MMainWindow::calcSelectedTime()
 			dt_end.setTime(_dateTimeEdit->timeEdit()->time());
 			break;
 	}
+
+	if (_delayType == DelayType_Now)
+		return i18n("No Delay");
+
+	if (_delayType == DelayType_OnAppExit)
+		return QString::null;
+
+	return MiscUtils::formatDateTime(dt_end.addSecs(1));
 }
 
 void MMainWindow::initMainMenu()
@@ -433,43 +440,140 @@ void MMainWindow::initMainMenu()
 
 	// file
 	KPopupMenu *pm_file = new KPopupMenu(this);
-	if (!MiscUtils::isRestricted("tab_stats"))
-	{
-		pm_file->insertItem(
-			i18n("&Statistics"),
-			this, SLOT(slotStats()), Key_F2
-		);
-		pm_file->insertSeparator();
-	}
-// TODO: 0.9: KActions
-// TODO: 0.9: Kiosk
+	_statisticsAction->plug(pm_file);
+	pm_file->insertSeparator();
+	if (MiscUtils::isRestricted("tab_stats"))
+		_statisticsAction->setEnabled(false);
+
+// TODO: 2.0: improve kiosk support
 	KStdAction::quit(this, SLOT(slotQuit()), actionCollection())->plug(pm_file);
 	mainMenu->insertItem(i18n("&File"), pm_file);
 
+	// actions
+	KPopupMenu *pm_actions = new KPopupMenu(this);
+	pm_actions->insertTitle(SmallIcon("messagebox_warning"), i18n("No Delay"));
+	_shutDownAction->plug(pm_actions);
+	_rebootAction->plug(pm_actions);
+	_lockScreenAction->plug(pm_actions);
+	_logoutAction->plug(pm_actions);
+	pm_actions->insertSeparator();
+	_cancelAction->plug(pm_actions);
+	mainMenu->insertItem(i18n("&Actions"), pm_actions);
+
 	// settings
 	KPopupMenu *pm_settings = new KPopupMenu(this);
-	pm_settings->insertItem(
-		SmallIcon("button_ok"), i18n("Check &System Configuration"),
-		this, SLOT(slotCheckSystemConfig())
-	);
+	_checkSystemConfigurationAction->plug(pm_settings);
+	pm_settings->insertSeparator();
+	pm_settings->insertItem(SmallIcon("configure_shortcuts"), i18n("Configure Global Shortcuts..."), this, SLOT(slotConfigureGlobalShortcuts()));
+	KStdAction::keyBindings(this, SLOT(slotConfigureShortcuts()), actionCollection())->plug(pm_settings);
+	pm_settings->insertSeparator();
 	KStdAction::configureNotifications(this, SLOT(slotConfigureNotifications()), actionCollection())->plug(pm_settings);
-	KStdAction::preferences(this, SLOT(slotConfigure()), actionCollection())->plug(pm_settings);
+	_configureKShutDownAction->plug(pm_settings);
 	mainMenu->insertItem(i18n("&Settings"), pm_settings);
 
 	// help
 	mainMenu->insertItem(SmallIcon("help"), i18n("&Help"), helpMenu());
+}
 
-	// init actions
-	new KAction(
-		i18n("&Cancel"), KShortcut(Key_Escape),
+void MMainWindow::initActions()
+{
+	// shut down actions
+
+	ac_shutDown = new KActionCollection(this, this);
+
+	_lockScreenAction = new KAction(
+		ks_actions->getName(Action::LockScreen),
+		ks_actions->getIcon(Action::LockScreen), KShortcut(),
+		ks_actions, SLOT(slotLockScreen()),
+		ac_shutDown, "kshutdown_lockscreen"
+	);
+
+	_logoutAction = new KAction(
+		ks_actions->getName(Action::Logout),
+		ks_actions->getIcon(Action::Logout), KShortcut(),
+		ks_actions, SLOT(slotLogout()),
+		ac_shutDown, "kshutdown_logout"
+	);
+
+	_rebootAction = new KAction(
+		ks_actions->getName(Action::Reboot),
+		ks_actions->getIcon(Action::Reboot), KShortcut(),
+		ks_actions, SLOT(slotReboot()),
+		ac_shutDown, "kshutdown_reboot"
+	);
+
+	_shutDownAction = new KAction(
+		ks_actions->getName(Action::ShutDown),
+		ks_actions->getIcon(Action::ShutDown), KShortcut(),
+		ks_actions, SLOT(slotShutDown()),
+		ac_shutDown, "kshutdown_shutdown"
+	);
+
+	// standard actions
+
+	_cancelAction = new KAction(
+		i18n("C&ancel"),
+		"cancel", KShortcut(Key_Escape),
 		this, SLOT(slotCancel()),
-		actionCollection(), "ks_cancel"
+		actionCollection(), "kshutdown_cancelaction"
 	);
-	new KAction(
-		i18n("&Start"), KShortcut(Key_Return),
-		this, SLOT(slotStartStop()),
-		actionCollection(), "ks_startstop"
+	_cancelAction->setEnabled(false);
+
+	_checkSystemConfigurationAction = new KAction(
+		i18n("Check &System Configuration"),
+		"button_ok", KShortcut(),
+		this, SLOT(slotCheckSystemConfig()),
+		actionCollection(), "kshutdown_checksystemconfiguration"
 	);
+
+	_configureKShutDownAction = KStdAction::preferences(this, SLOT(slotConfigureKShutDown()), actionCollection());
+
+	_startAction = new KAction(
+		i18n("&Start"),
+		"button_ok", KShortcut(Key_Return),
+		this, SLOT(slotStart()),
+		actionCollection(), "kshutdown_startaction"
+	);
+	MiscUtils::plug(_startAction, b_startStop);
+
+	_statisticsAction = new KAction(
+		i18n("&Statistics"),
+		KShortcut(Key_F2),
+		this, SLOT(slotStats()),
+		actionCollection(), "kshutdown_statistics"
+	);
+
+	actionCollection()->readShortcutSettings();
+	ac_shutDown->readShortcutSettings();
+
+	// init global actions/shortcuts
+	_globalAccel = new KGlobalAccel(this);
+	_globalAccel->insert(
+		"kshutdown_shutdown",
+		ks_actions->getName(Action::ShutDown),
+		QString(),
+		0, 0, ks_actions, SLOT(slotShutDown())
+	);
+	_globalAccel->insert(
+		"kshutdown_reboot",
+		ks_actions->getName(Action::Reboot),
+		QString(),
+		0, 0, ks_actions, SLOT(slotReboot())
+	);
+	_globalAccel->insert(
+		"kshutdown_lockscreen",
+		ks_actions->getName(Action::LockScreen),
+		QString(),
+		0, 0, ks_actions, SLOT(slotLockScreen())
+	);
+	_globalAccel->insert(
+		"kshutdown_logout",
+		ks_actions->getName(Action::Logout),
+		QString(),
+		0, 0, ks_actions, SLOT(slotLogout())
+	);
+	_globalAccel->readSettings();
+	_globalAccel->updateConnections();
 }
 
 QWidget *MMainWindow::initWidgets()
@@ -531,10 +635,7 @@ QWidget *MMainWindow::initWidgets()
 	b_startStop = new KPushButton(w, "KPushButton::b_startStop");
 	b_startStop->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed));
 	b_startStop->setDefault(true); // enter key
-	b_startStop->setIconSet(SmallIcon("button_ok"));
-	b_startStop->setText(i18n("Start [Enter]"));
 	l->addWidget(b_startStop);
-	connect(b_startStop, SIGNAL(clicked()), SLOT(slotStartStop()));
 
 	return w;
 }
@@ -546,17 +647,16 @@ void MMainWindow::setCaptions(const QString &remainingTime, const QString &selec
 	if (ks_actions->active())
 	{
 		if (remainingTime.isNull())
-			s = selectedTime + " - " + ks_actions->getCurrentName();
+			s = selectedTime + " > " + ks_actions->getCurrentName();
 		else
-			s = remainingTime + " / " + selectedTime + " - " + ks_actions->getCurrentName();
+			s = remainingTime + " > " + selectedTime + " - " + ks_actions->getCurrentName();
 	}
 	if (ks_actions->testMode())
 		s += " [" + i18n("TEST MODE") + "]";
 	setCaption(s);
 
 	// system tray tool tip
-	if (ks_actions->active())
-	{
+	if (ks_actions->active()) {
 		s = "<br>";
 		s += i18n("Remaining time: <b>%1</b>").arg(remainingTime.isNull() ? i18n("Unknown") : remainingTime) + "<br>";
 		s += i18n("Selected time: <b>%1</b>").arg(selectedTime) + "<br>";
@@ -566,20 +666,19 @@ void MMainWindow::setCaptions(const QString &remainingTime, const QString &selec
 			s += "<br>" + i18n("<b>Note: </b> The test mode is enabled") + "<br>";
 		}
 	}
-	else
-	{
+	else {
 		s = "";
 	}
-	if (MSystemTray::isInstance())
-	{
+	if (MSystemTray::isInstance()) {
 		QToolTip::add(
 			ks_tray,
-			"<qt>" \
-			"<b>KShutDown</b><br>" +
-			s +
-			"<br>" +
-			i18n("<b>Tip:</b> Use the <b>Middle Mouse Button</b> to display the actions menu") +
-			"</qt>"
+			MiscUtils::HTML("<b>KShutDown</b><br>" + s)
+		);
+	}
+	if (ProgressBar::isInstance()) {
+		QToolTip::add(
+			ProgressBar::getInstance(),
+			MiscUtils::HTML("<b>KShutDown</b><br>" + s)
 		);
 	}
 }
@@ -591,6 +690,14 @@ void MMainWindow::setWidgetsEnabled(const bool yes)
 	cb_actions->setEnabled(yes);
 	ks_extras->button()->setEnabled(yes);
 	_appObserver->setWidgetsEnabled(yes);
+
+	// actions
+	_cancelAction->setEnabled(!yes);
+	_startAction->setEnabled(yes);
+	if (yes)
+		MiscUtils::plug(_startAction, b_startStop);
+	else
+		MiscUtils::plug(_cancelAction, b_startStop);
 }
 
 void MMainWindow::updateTimeInfo(const int secs)
@@ -599,6 +706,12 @@ void MMainWindow::updateTimeInfo(const int secs)
 		MiscUtils::formatDateTime(secs),
 		KGlobal::locale()->formatDateTime(dt_end, true, true)
 	);
+
+	// update progress bar
+	if (ProgressBar::isInstance()) {
+		ProgressBar *progressBar = ProgressBar::getInstance();
+		progressBar->setProgress(secs);
+	}
 }
 
 void MMainWindow::slotCancel()
@@ -606,6 +719,10 @@ void MMainWindow::slotCancel()
 	cancel();
 }
 
+void MMainWindow::slotStart() {
+	start(true);
+}
+
 void MMainWindow::slotQuit()
 {
 	ks_actions->setTotalExit(true); // do not hide window in the system tray
@@ -629,7 +746,8 @@ void MMainWindow::slotCheckTime()
 			if (kshutdownrc->warningMessageEnabled)
 			{
 				cancel();
-				messageDialog(30);
+				if (MMessageDialog::show(30))
+					ks_actions->execCurrent();
 			}
 			else
 			{
@@ -675,7 +793,8 @@ void MMainWindow::slotCheckTime()
 		makeVisible();
 		// show warning message dialog
 		cancel();
-		messageDialog(secsTo);
+		if (MMessageDialog::show(secsTo))
+			ks_actions->execCurrent();
 
 		return;
 	}
@@ -683,22 +802,38 @@ void MMainWindow::slotCheckTime()
 	MiscUtils::notifyUser(secsTo);
 }
 
-void MMainWindow::slotDelayTypeChange(int index)
-{
+void MMainWindow::slotDelayTypeChange(int index) {
 	setDelayType((DelayType)index);
 	slotUpdateTimeInfo();
 }
 
-void MMainWindow::slotConfigure()
-{
+void MMainWindow::slotConfigureGlobalShortcuts() {
+	if (KKeyDialog::configure(_globalAccel, false, this)) {
+		_globalAccel->writeSettings(0, true);
+		_globalAccel->updateConnections();
+	}
+}
+
+void MMainWindow::slotConfigureKShutDown() {
 	configure();
 }
 
-void MMainWindow::slotConfigureNotifications()
-{
+void MMainWindow::slotConfigureNotifications() {
 	KNotifyDialog::configure(this);
 }
 
+void MMainWindow::slotConfigureShortcuts()
+{
+	KKeyDialog *dialog = new KKeyDialog(false, this);
+	dialog->insert(actionCollection());
+	dialog->insert(ac_shutDown);
+	dialog->configure(true);
+	delete dialog;
+
+	actionCollection()->writeShortcutSettings();
+	ac_shutDown->writeShortcutSettings();
+}
+
 void MMainWindow::slotActionChange(int index)
 {
 	setAction((Action::Type)(index + 1)); // 0 = What_Nothing
@@ -709,40 +844,15 @@ void MMainWindow::slotCheckSystemConfig()
 	SystemConfig::check(this);
 }
 
-void MMainWindow::slotStartStop()
-{
-	if (ks_actions->active())
-		cancel();
-	else
-		start();
-}
-
 void MMainWindow::slotStats()
 {
-	if (MiscUtils::isRestricted("tab_stats"))
-		return;
-
-	if (!_stats)
-		_stats = new MStatsTab(this);
-	_stats->update();
-	_stats->show();
-	_stats->raise();
+	if (!MiscUtils::isRestricted("tab_stats"))
+		MStatsTab::getInstance()->show();
 }
 
 void MMainWindow::slotUpdateTimeInfo()
 {
-	calcSelectedTime();
-	QString s;
-	if (_delayType == DelayType_Now)
-		s = i18n("No delay");
-	else if (_delayType == DelayType_OnAppExit)
-		s = QString::null;
-	else
-		s = MiscUtils::formatDateTime(dt_end.addSecs(1));
-	if (s.isNull())
-		l_statusLine->setText(s);
-	else
-		l_statusLine->setText("<b>" + s + "</b>");
+	l_statusLine->setText("<b>" + getTimeInfo() + "</b>");
 }
 
 bool MMainWindow::queryClose()
@@ -754,7 +864,13 @@ bool MMainWindow::queryClose()
 	// MSystemTray::Always
 	if (kshutdownrc->systemTray == MSystemTray::Always)
 	{
-// TODO: 0.9: display message box
+		if (_showMinimizeInfo) {
+			MiscUtils::passiveMessage(
+				i18n("KShutDown has been minimized"),
+				ks_tray
+			);
+			_showMinimizeInfo = false;
+		}
 		ks_tray->flashIcon();
 		hide();
 
@@ -766,18 +882,33 @@ bool MMainWindow::queryClose()
 	{
 		if (ks_actions->active())
 		{
+			if (_showMinimizeInfo) {
+				MiscUtils::passiveMessage(
+					i18n("KShutDown has been minimized"),
+					ks_tray
+				);
+				_showMinimizeInfo = false;
+			}
 			ks_tray->flashIcon();
 			hide();
 
 			return false;
 		}
+		else {
+// FIXME: 2.0: quit message
+			MiscUtils::passiveMessage(i18n("KShutDown has quit"));
 
-		return true;
+			return true;
+		}
 	}
 
 	// MSystemTray::Never
-	if (kshutdownrc->systemTray == MSystemTray::Never)
+	if (kshutdownrc->systemTray == MSystemTray::Never) {
+// FIXME: 2.0: quit message
+		MiscUtils::passiveMessage(i18n("KShutDown has quit"));
+
 		return true;
+	}
 
 	return true;
 }
