@@ -46,22 +46,23 @@ public:
 	 * Destructor.
 	 */
 	virtual ~MMessageDialog();
+
+	static void cancel();
+	static bool show(const int timeout);
 private:
 	int
 		_dialogDelay,
 		_lastTime;
 	Action::Type _action;
 	KPushButton *b_continue;
+	static MMessageDialog *_instance;
 	QLCDNumber *l_seconds;
 	QTimer *_checkTimer;
-public slots:
-	/**
-	 * Cancels the active action.
-	 */
-	void slotCancel();
 private slots:
-	void slotCheck();
-	void slotContinue();
+	void slotAccept();
+	void slotEnableContinue();
+	void slotCheckTime();
+	void slotReject();
 };
 
 #endif // __MMESSAGEDIALOG_H__
