@@ -29,7 +29,6 @@
 #include "systemconfig.h"
 
 #include <qcheckbox.h>
-#include <qcombobox.h>
 #include <qfile.h>
 #include <qhbox.h>
 #include <qhbuttongroup.h>
@@ -47,6 +46,7 @@
 #include <klistview.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <kpopupmenu.h>
 #include <knuminput.h>
 #include <kpushbutton.h>
 #include <kstandarddirs.h>
@@ -54,6 +54,7 @@
 #include <kurllabel.h>
 
 int MSettingsDialog::_lastPage = 0;
+MSettingsDialog *MSettingsDialog::_instance = 0;
 
 // public
 
@@ -69,6 +70,8 @@ MSettingsDialog::MSettingsDialog(QWidget *parent)
 		true // separator
 	)
 {
+	_instance = this;
+
 	initGeneralPage();
 	initMessagesPage();
 	initActionsPage();
@@ -86,6 +89,8 @@ MSettingsDialog::MSettingsDialog(QWidget *parent)
 
 	setInitialSize(configDialogSize("Settings Dialog"));
 	showPage(_lastPage); // restore last page
+
+	exec();
 }
 
 MSettingsDialog::~MSettingsDialog()
@@ -191,8 +196,8 @@ void MSettingsDialog::initAdvancedPage()
 	QCheckBox *c_cdTrayClose = new QCheckBox(i18n("Close CD-ROM Tray"), gb_cdTrayClose, "kcfg_cd_tray_close");
 	QHBox *cdTrayCloseBox = new QHBox(gb_cdTrayClose);
 	QLabel *l_cdTrayCloseCommand = new QLabel(i18n("Command:"), cdTrayCloseBox);
-// TODO: 0.9: disable in_cdTrayCloseCommand if c_cdTrayClose is not checked
-// TODO: 0.9: "Test" button
+// FIXME: 2.0: disable "in_cdTrayCloseCommand" if "c_cdTrayClose" is not checked
+// TODO: 2.0: "Test" button
 	KLineEdit *in_cdTrayCloseCommand = new KLineEdit(cdTrayCloseBox, "kcfg_cd_tray_close_command");
 	l_cdTrayCloseCommand->setBuddy(in_cdTrayCloseCommand);
 	connect(
@@ -244,19 +249,29 @@ void MSettingsDialog::initGeneralPage()
 	l->addWidget(gb_systemTray);
 
 	// karamba info
-	QString karambaThemes = locate("appdata", "karamba/kshutdown-32x32.png");
+	QString karambaThemes = locate("appdata", "karamba/kshutdown-48x48.png");
 	if (!karambaThemes.isNull())
 	{
 		int i = karambaThemes.findRev("/");
 		if ((i != -1) && (i > 0))
 		{
 			MTip *t_karamba = new MTip(MTip::Info, _generalPage);
+/* TODO: 2.0: superkaramba integration
+			t_karamba->disconnect(
+				t_karamba, SIGNAL(linkClicked(const QString &)),
+				t_karamba, SLOT(openLink(const QString &))
+			);
+			t_karamba->connect(
+				t_karamba, SIGNAL(linkClicked(const QString &)),
+				this, SLOT(slotOpenKarambaLink(const QString &))
+			);
+*/
 			karambaThemes = karambaThemes.left(i);
 			t_karamba->setTipText(
 				// title
 				"<h2>SuperKaramba</h2>" \
 				// themes
-				"<a href=\"" + karambaThemes + "\">" + i18n("KShutDown Themes") + "</a><br>" \
+				"<a href=\"" + karambaThemes + "\">" + i18n("Show KShutDown Themes") + "</a><br>" \
 				// home page
 				"<a href=\"http://www.superkaramba.com/index.php\">" + i18n("SuperKaramba Home Page") + "</a>" \
 			);
@@ -279,12 +294,18 @@ void MSettingsDialog::initMessagesPage()
 	// enabled/info
 	QCheckBox *c_warningMessageEnabled = new QCheckBox(i18n("Display a warning message before action"), w_warning, "kcfg_warning_message_enabled");
 	// slider
-	KIntNumInput *in_warningMessageDelay = new KIntNumInput(w_warning, "kcfg_warning_message_delay");
+	in_warningMessageDelay = new KIntNumInput(w_warning, "kcfg_warning_message_delay");
 	in_warningMessageDelay->setSuffix(" " + i18n("minute(s)"));
 	in_warningMessageDelay->setRange(1, 60, 1, true);
-	KPushButton *b_warningMessageTest = new KPushButton(i18n("Test"), w_warning, "KPushButton::b_warningMessageTest");
+	KPushButton *b_warningMessageTest = new KPushButton(KStdGuiItem::test(), w_warning, "KPushButton::b_warningMessageTest");
 	connect(b_warningMessageTest, SIGNAL(clicked()), SLOT(slotWarningMessageTest()));
+
+	QLabel *l_warningText = new QLabel(i18n("Recommended"), w_warning);
+	l_warningText->setFont(QFont(l_warningText->font().family(), l_warningText->font().pointSize(), QFont::Bold));
+	l_warning->addWidget(l_warningText);
+
 	l_warning->addWidget(c_warningMessageEnabled);
+
 	l_warning->addWidget(in_warningMessageDelay);
 	l_warning->addStretch();
 	l_warning->addWidget(b_warningMessageTest);
@@ -295,42 +316,53 @@ void MSettingsDialog::initMessagesPage()
 	QVBoxLayout *l_customMessage = new QVBoxLayout(w_customMessage, 5);
 	QCheckBox *c_customMessageEnabled = new QCheckBox(i18n("Enabled"), w_customMessage, "kcfg_custom_message_enabled");
 	QLabel *l_customMessageCommand = new QLabel(i18n("A shell command to execute:"), w_customMessage);
-	KLineEdit *in_customMessageCommand = new KLineEdit(w_customMessage, "kcfg_custom_message_command");
-	MiscUtils::setHint(in_customMessageCommand,
-		"<qt>" +
-		i18n("Enter a command.") +
-		"<ul>" \
-		"<li><b>%appname</b> KShutDown</li>" \
-		"<li><b>%text</b> " + i18n("A message text") + "</li>" \
-		"<li><b>%title</b> " + i18n("The current main window title") + "</li>" \
-		"</ul>"
-		"</qt>"
+	in_customMessageCommand = new KLineEdit(w_customMessage, "kcfg_custom_message_command");
+	MiscUtils::setHint(
+		in_customMessageCommand,
+		MiscUtils::HTML(
+			i18n("Enter a command.") +
+			"<ul>" \
+				"<li><b>%appname</b> KShutDown</li>" \
+				"<li><b>%text</b> " + i18n("A message text") + "</li>" \
+				"<li><b>%title</b> " + i18n("The current main window title") + "</li>" \
+			"</ul>"
+		)
 	);
 	l_customMessageCommand->setBuddy(in_customMessageCommand);
 
-	KPushButton *b_testCustomMessage = new KPushButton(i18n("Test"), w_customMessage, "KPushButton::b_testCustomMessage");
+	KPushButton *b_customMessagePresets = new KPushButton(i18n("Presets"), w_customMessage, "KPushButton::b_customMessagePresets");
+	KPopupMenu *pm_customMessagePresets = new KPopupMenu(b_customMessagePresets);
+	pm_customMessagePresets->insertItem("KDialog", this, SLOT(slotSetKDialogCustomMessage()));
+	pm_customMessagePresets->insertItem("Amor", this, SLOT(slotSetAmorCustomMessage()));
+	b_customMessagePresets->setPopup(pm_customMessagePresets);
+
+	KPushButton *b_testCustomMessage = new KPushButton(KStdGuiItem::test(), w_customMessage, "KPushButton::b_testCustomMessage");
 	connect(b_testCustomMessage, SIGNAL(clicked()), SLOT(slotCustomMessageTest()));
 	l_customMessage->addWidget(c_customMessageEnabled);
 	l_customMessage->addWidget(l_customMessageCommand);
 	l_customMessage->addWidget(in_customMessageCommand);
+	l_customMessage->addWidget(b_customMessagePresets);
 	l_customMessage->addStretch();
 	l_customMessage->addWidget(b_testCustomMessage);
 	t_messages->addTab(w_customMessage, i18n("Custom Message"));
 
+	// progress bar
+	QVGroupBox *w_progressBar = new QVGroupBox(i18n("Progress Bar"), _messagesPage);
+	new QCheckBox(i18n("Enabled"), w_progressBar, "kcfg_progress_bar_enabled");
+
 	// re-enable messages
 	b_enableAllMessages = new KPushButton(i18n("Re-enable All Message Boxes"), _messagesPage, "KPushButton::b_enableAllMessages");
 	MiscUtils::setHint(
 		b_enableAllMessages,
-		"<qt>" +
-		i18n(
-		"Enable all messages which have been turned off with the " \
-		"<b>Do not show this message again</b> feature."
-		 ) +
-		"</qt>"
+		MiscUtils::HTML(i18n(
+			"Enable all messages which have been turned off with the " \
+			"<b>Do not show this message again</b> feature."
+		))
 	);
 	connect(b_enableAllMessages, SIGNAL(clicked()), SLOT(slotEnableAllMessages()));
 
 	l->addWidget(t_messages);
+	l->addWidget(w_progressBar);
 	l->addStretch();
 	l->addWidget(b_enableAllMessages);
 
@@ -343,6 +375,9 @@ void MSettingsDialog::initMessagesPage()
 	connect(
 		c_customMessageEnabled, SIGNAL(toggled(bool)),
 		in_customMessageCommand, SLOT(setEnabled(bool)));
+	connect(
+		c_customMessageEnabled, SIGNAL(toggled(bool)),
+		b_customMessagePresets, SLOT(setEnabled(bool)));
 	connect(
 		c_customMessageEnabled, SIGNAL(toggled(bool)),
 		b_testCustomMessage, SLOT(setEnabled(bool)));
@@ -409,7 +444,7 @@ void MSettingsDialog::updateItem(KListViewItem *item)
 			if (pause == 0)
 				cmd += " [" + i18n("No pause") + "]";
 			else
-				cmd += " [" + i18n("Pause:") + " " + QString::number(pause) + "]";
+				cmd += " [" + i18n("Pause: %1").arg(QString::number(pause) + "s") + "]";
 			item->setText(2, cmd);
 		}
 		else
@@ -433,7 +468,7 @@ void MSettingsDialog::slotConfigChanged()
 
 void MSettingsDialog::slotCustomMessageTest()
 {
-	MiscUtils::customMessage(i18n("Test"), kshutdownrc->customMessageCommand);
+	MiscUtils::customMessage(i18n("Test"), in_customMessageCommand->text());
 }
 
 void MSettingsDialog::slotEditAction()
@@ -468,13 +503,17 @@ void MSettingsDialog::slotOKClicked()
 	saveDialogSize("Settings Dialog");
 	_configDialogManager->updateSettings();
 
+// FIXME: 2.0: apply button
+
 	// lock after login
 	if (c_autoLock->isChecked())
 	{
 		Links::createLink(
 			KGlobalSettings::autostartPath() + "/kshutdown-autolock.desktop",
 			"kshutdown --init --lock",
-			"kshutdown"
+			"kshutdown",
+			"KShutDown",
+			i18n("This file is used to lock session at KDE startup")
 		);
 	}
 	else
@@ -491,6 +530,16 @@ void MSettingsDialog::slotOKClicked()
 	slotConfigChanged();
 }
 
+// TODO: 2.0: remember last page
+
+void MSettingsDialog::slotSetAmorCustomMessage() {
+	in_customMessageCommand->setText(KS_CONFIG_AMOR_MESSAGE);
+}
+
+void MSettingsDialog::slotSetKDialogCustomMessage() {
+	in_customMessageCommand->setText(KS_CONFIG_KDIALOG_MESSAGE);
+}
+
 void MSettingsDialog::slotShutDownProblem()
 {
 	showPage(pageIndex(_actionsPage));
@@ -504,7 +553,7 @@ void MSettingsDialog::slotTrayMessageProblem()
 void MSettingsDialog::slotWarningMessageTest()
 {
 	MMessageDialog *messageDialog = new MMessageDialog(
-		kshutdownrc->warningMessageDelay * 60,
+		in_warningMessageDelay->value() * 60,
 		Action::Nothing,
 		i18n("Test")
 	);
